#!/usr/bin/env python
#
# Copyright (c) 2016-2017, Parallels International GmbH
#
# This file is part of disp-helper. It's free software; you can redistribute
# it and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the License,
# or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
# Our contact details: Parallels International GmbH, Vordergasse 59, 8200
# Schaffhausen, Switzerland.
#

import time
import libvirt
from infra_cep_helper import CEPCollecting
from xml.etree import ElementTree as ET
from xml.dom import minidom

class libvirt_get_info(CEPCollecting):
    def get_stats(self):
        r = dict()
        r["Total_amount_of_VMs"] = len(self.conn.listAllDomains())
        # active it's not "running" is vms in state_names
        r["Total_amount_of_active_VMs"] = len(self.conn.listDomainsID())
        # 0 domains if state_names return nothing
        r["Total_amount_of_inactive_VMs"] = len(self.conn.listDefinedDomains())

        if self.is_exist('openstack-nova-compute') and self.service_active('openstack-nova-compute'):
            r["CLUSTER_COMPUTE"] = "yes"

        host_nets = self.conn.listNetworks()
        if len ( host_nets ) > 0:
           for network in host_nets:
               r["host_network:" + str(network)] = self.get_info_about_networks(self.conn, network)

        active_hosts = self.conn.listDomainsID()
        if  len ( active_hosts ) > 0 :
            for id in active_hosts:
                r["DOMAIN_INFO_FOR_id_" + str(id)] = self.check_info_for_domain(self.conn, id)
        return r

    def createConnection(self):
        self.conn = libvirt.openReadOnly(None)
        if self.conn == None:
           print 'Failed to open connection to QEMU/KVM'
           sys.exit(1)
        else:
            return self.conn

    def look_at_xml(self, dom, tag, element):
        raw_xml = dom.XMLDesc(0)
        xml = minidom.parseString(raw_xml)
        list = []
        interfaceTypes = xml.getElementsByTagName(tag)
        for interfaceType in interfaceTypes:
            interfaceNodes = interfaceType.childNodes
            attrs = {}
            for interfaceNode in interfaceNodes:
                if interfaceNode.nodeName[0:1] != '#':
                    if isinstance(element, basestring):
                        if interfaceNode.attributes.has_key(element):
                           list.append(interfaceNode.attributes[element].value)
                    else:
                        for key in element:
                            if interfaceNode.attributes.has_key(key):
                                attrs[key] = interfaceNode.attributes[key].value
            if len(attrs.keys()) > 0:
                list.append(attrs)
        return list

    def get_info_about_networks(self, conn, network):
        net_dict = dict()
        network = conn.networkLookupByName(network)
        uuid = network.UUIDString()
        network = conn.networkLookupByUUIDString(uuid)
        net_dict["NETWORK_NAME"] = network.name()
        net_dict["NETWORK_UUID"] = network.UUIDString()
        net_dict["NETWORK_BRIDGE"] = network.bridgeName()
        net_dict["NETWORK_AUTOSTART"] = str(network.autostart())
        net_dict["NETWORK_ACTIVE"] = str(network.isActive())
        net_dict["NETWORK_PERSISTENT"] = str(network.isPersistent())
        return net_dict

    def check_info_for_domain(self, conn, id):
        # digits to human-readable states
        state_names = {  libvirt.VIR_DOMAIN_NOSTATE     : "no state",
                         libvirt.VIR_DOMAIN_RUNNING     : "running",
                         libvirt.VIR_DOMAIN_BLOCKED     : "idle",
                         libvirt.VIR_DOMAIN_PAUSED      : "paused",
                         libvirt.VIR_DOMAIN_SHUTDOWN    : "in shutdown",
                         libvirt.VIR_DOMAIN_SHUTOFF     : "shut off",
                         libvirt.VIR_DOMAIN_CRASHED     : "crashed",
                         libvirt.VIR_DOMAIN_PMSUSPENDED : "suspended by PM" }

        vm_dict = dict()
        dom = self.conn.lookupByID(id)
        infos = dom.info()
        vm_dict["DOMAIN_NAME"] = dom.name() 
        vm_dict["DOMAIN_MAX_MEM"] = infos[1]
        vm_dict["DOMAIN_STATE"] = state_names[infos[0]]
        vm_dict["DOMAIN_MAX_MEMORY"] = infos[1]
        vm_dict["DOMAIN_VIRT_CPUs"] = infos[3]
        vm_dict["DOMAIN_CPU_TIME"] = infos[2]
        model = conn.getSecurityModel()
        type = dom.OSType()
        vm_dict["SECURITY_MODEL"] = model
        vm_dict["OS_TYPE"] = type

        disk = self.look_at_xml(dom, 'disk', ['dev', 'file'])

        netdev = self.look_at_xml(dom, 'interface', ['dev', 'address'])
        netbr = self.look_at_xml(dom, 'interface', ['dev', 'bridge'])
        nettype = self.look_at_xml(dom, 'interface', ['dev', 'type'])

        vm_dict["ETHER_TYPE"] = self.look_at_xml(dom, 'interface', 'type')
        for el in netdev:
            vm_dict["MAC_ADDRESS_" + el['dev']] = el['address'] if 'dev' in el else ''
        for el in netbr:
            vm_dict["BRIDGE_" + el['dev']] = el['bridge'] if 'dev' in el else ''
        for el in nettype:
            vm_dict["ETHER_TYPE_" + el['dev']] = el['type'] if 'dev' in el else ''
        for el in disk:
            vm_dict["DISK_DEVICE_" + el['dev']] = el['file'] if 'file' in el else ''

        return vm_dict

    def fill_results(self):
        retries = 0
        succeed = False
        fail_reason = "Unknown reason"
        while not succeed and retries < 1:
            try:
                self.conn = libvirt.openReadOnly(None)
                if self.conn is None:
                    raise Exception("Unable to connect to libvirt")
                self.result["libvirtINFO"] = self.get_stats()
                succeed = True
            except Exception as e:
                retries += 1
                time.sleep(1 * 1 * retries)
                succeed = False
                fail_reason = str(e)
        if not succeed:
            raise StandardError("Unable to collect libvirt stats info: " + fail_reason)

def main():
    o = libvirt_get_info()
    o.run()

if __name__ == "__main__":
    main()
