#!/usr/bin/env python
#
# Copyright (c) 2016-2017, Parallels International GmbH
#
# This file is part of disp-helper. It's free software; you can redistribute
# it and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the License,
# or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
# Our contact details: Parallels International GmbH, Vordergasse 59, 8200
# Schaffhausen, Switzerland.
#

from infra_cep_helper import CEPCollecting
from lxml import etree
import os
import json
import xmltodict
import subprocess

class Storage(CEPCollecting):
    def fill_results(self):
        l = list()
        if not os.path.isdir("/etc/vstorage/clusters"):
            self.result['StorageClusters'] = l
            return
        clusters = os.listdir('/etc/vstorage/clusters')
        for cluster_name in clusters:
            cluster_stats = self.execute_shell("vstorage -c " + cluster_name + " stat -X").strip()
            root = etree.fromstring(cluster_stats)

            cluster_info = {"Name": cluster_name,
                            "ID"     : root.find('./cluster_id').text,
                            "MDS"    : len(root.findall('./mds_list/mds')),
                            "CS"     : len(root.findall('./cs_list/cs')),
                            "Clients": len(root.findall('./clients_list/client')),
                            "Hosts"  : len(root.findall('./host_list/host_stat')),
                            "Repl_n" : root.find('./repl/norm').text,
                            "Repl_m" : root.find('./repl/max').text,
                            "Repl_l" : root.find('./repl/limit').text,
                            "Alloc"  : root.find('./space/allocatable').text,
                            "Eff"    : root.find('./space/effective_total').text,
                            "Alloc_r": root.find('./space/allocatable_raw').text,
                            "Total"  : root.find('./space/total').text,
                            "Total_r": root.find('./space/total_raw').text,
                            "Free"   : root.find('./space/free').text
                        }
            l.append(cluster_info)
            # run vstorage get-events and get-config only if cluster exist
            self.result["Vstorage_Config_" + cluster_name] = self.get_cfg_output(cluster_name)
            self.result["Event_" + cluster_name] = self.get_events_output(cluster_name)
            self.result["License_" + cluster_name] = self.get_license_output(cluster_name)
            self.result["Stats_" + cluster_name] = self.get_stats_output(cluster_name)
            # pull psql tables only if postgresql is running
	    service_activated = self.service_active('postgresql')
            if service_activated:
               table_names = ('alerts', 'ifaces', 'iface_roles', 'users', 'iscsi_targets', 'iscsi_luns', 'nfs_shares', 'nfs_exports', 's3_clusters', 's3_nodes', 'snmp_settings')
               for table in table_names:
                   self.result["check_databases_" + cluster_name + "_" + table] = self.get_database_output(cluster_name, table)
	self.result["StorageClusters"] = l

    def get_cfg_output(self, cluster_name):
        value = ""
        key = ""
        cfg_out = {}

        try:
            cluster_config = self.execute_shell("/usr/bin/vstorage -c " + cluster_name + " get-config")
            for l in cluster_config.splitlines():
                if "=" in l:
                    if key:
                        cfg_out[key] = value
                    l_array = l.strip().split("=")
                    key = l_array[0].strip()
                    value = ":".join(l_array[1:]).strip()
                else:
                    value += "" + l.strip()
            if key:
                cfg_out[key] = value.strip()
        except:
            return {}
        return cfg_out

    def get_events_output(self, cluster_name):
        value = ""
        key = ""
        event_out = {}

        try:
            out = self.execute_shell("/usr/bin/vstorage -c " + cluster_name + " get-event")
            for l in out.splitlines():
                if l.startswith("connected"):
                    continue
                if ": " in l:
                    if key:
                        event_out[key] = value
                    l_array = l.strip().split(": ")
                    key = l_array[0].strip()
                    value = ":".join(l_array[1:]).strip()
                else:
                    value += " " + l.strip()
            if key:
                event_out[key] = value.strip()
        except:
            return {}
        return event_out

    def get_stats_output(self, cluster_name):
	try:
	    out = subprocess.Popen(["/usr/bin/vstorage", "-c", cluster_name, "stat", "--xml"], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
	    (output, err) = out.communicate()
            xmlString=output
	    jsonString = json.dumps(xmltodict.parse(xmlString), indent=4)
	    status_json = json.loads(jsonString)
	except:
	    return {}
        return status_json

    def get_database_output(self, cluster_name, table):
        value = ""
        key = ""
        event_out = {}
        out = self.execute_shell('sudo -u vstoradmin /bin/psql -tAc \"select * from %s\"' %table)
        for l in out.splitlines():
            if "" in l:
                if key:
                   event_out[key] = value
                l_array = l.strip().split("|")
                key = l_array[0].strip('')
                value = ":".join(l_array[1:]).strip('')
        if key:
            event_out[key] = value.strip()
        return event_out

    def get_license_output(self, cluster_name):
        value = ""
        key = ""
        lic_out = {}

        try:
            out = self.execute_shell("/usr/bin/vstorage -c " + cluster_name+ " view-license")
            for l in out.splitlines():
                if "=" in l:
                    if key:
                        lic_out[key] = value
                    l_array = l.strip().split('=')
                    key = l_array[0].strip()
                    value = ":".join(l_array[1:]).strip('"')
                else:
                    value += ' ' + l.strip()
            if key:
                lic_out[key] = value.strip()
        except:
            return {}
        return lic_out

def main():
    o = Storage()
    o.run()


if __name__ == "__main__":
    main()
